#include "hdrToLMSR.h"
#include "utilities.h"
#include "tonemapping.h"
#include "float.h"

using namespace cv;
using namespace std; 

/*
void ReadLMSR(const char* filename, Mat* LMtx, Mat* MMtx, Mat* SMtx, Mat* RMtx)
{
  FILE *f = fopen(filename, "rb");
  float * buffer;
  int rows, cols;

  if (f == NULL) {fputs ("File error", stderr); exit(1);}
  fscanf(f,"%d\n%d\n",&cols,&rows);
  buffer = new float[rows*cols*4];
  fread(buffer,sizeof(float),rows*cols*4,f);
  printf("Load LMSR image done\n");
  fclose(f);

  for(int i=0;i<rows;i++) 
  {
    for(int j=0;j<cols;j++) 
    {
      int pixIdx = j+(i*cols); 

      LMtx->at<float>(i,j) = buffer[pixIdx*4+0];
      MMtx->at<float>(i,j) = buffer[pixIdx*4+1];
      SMtx->at<float>(i,j) = buffer[pixIdx*4+2];
      RMtx->at<float>(i,j) = buffer[pixIdx*4+3];
    }
  }
}
*/


void HDRToLMSR(Mat* hdrMat, Mat* LMtx, Mat* MMtx, Mat* SMtx, Mat* RMtx)
{
  float * buffer;
  int rows,cols;
  rows = hdrMat->rows;
  cols = hdrMat->cols;

  //read in buffers
  buffer = new float[rows*cols*3];
  for (int h = 0; h<rows; h++)
  {
    for (int w = 0; w < cols; w++)
    {
      int pixIdx = w+(h*cols);
      buffer[pixIdx*3 + 0] = hdrMat->at<Vec3f>(h,w)[2];  
      buffer[pixIdx*3 + 1] = hdrMat->at<Vec3f>(h,w)[1];  
      buffer[pixIdx*3 + 2] = hdrMat->at<Vec3f>(h,w)[0];  
    }
  }

  //for blending with final output.
  //WriteExrRGB("Result/tonemappedHDR.exr",buffer,cols,rows);
  //int i = system("./../TMlib/FredoToneMap/tone_mapping Result/tonemappedHDR.exr Result/tonemappedHDR.ppm 50.0");  
  ////printf("system call returned %d\n", i);

  //create convert matrix
  Mat ConvertMtx(4,3,CV_32FC1);

  #if 0
  //without white balance
  ConvertMtx.at<float>(0,0) = 0.023287629229077;
  ConvertMtx.at<float>(0,1) = 0.031761158107049;
  ConvertMtx.at<float>(0,2) = -0.065877379943775;
  ConvertMtx.at<float>(1,0) = 0.017477925227959;
  ConvertMtx.at<float>(1,1) = 0.025045276134633;
  ConvertMtx.at<float>(1,2) = -0.049961881755050;
  ConvertMtx.at<float>(2,0) = 0.005286774425047;
  ConvertMtx.at<float>(2,1) = 0.011053248400830;
  ConvertMtx.at<float>(2,2) = -0.014602394968991;
  ConvertMtx.at<float>(3,0) = 0.014179531162352;
  ConvertMtx.at<float>(3,1) = 0.023900602623377;
  ConvertMtx.at<float>(3,2) = -0.041615950732866;
  #endif

  #if 0
  //applied white balance on HDR training images 
  ConvertMtx.at<float>(0,0) = 0.053423145735707;
  ConvertMtx.at<float>(0,1) = 0.054289560383423;
  ConvertMtx.at<float>(0,2) = -0.061136577556301;
  ConvertMtx.at<float>(1,0) = 0.039629141289410;
  ConvertMtx.at<float>(1,1) = 0.043169600891946;
  ConvertMtx.at<float>(1,2) = -0.046254335314552;
  ConvertMtx.at<float>(2,0) = 0.007918710531123;
  ConvertMtx.at<float>(2,1) = 0.020833170138768;
  ConvertMtx.at<float>(2,2) = -0.011451393239790;
  ConvertMtx.at<float>(3,0) = 0.029447978916113;
  ConvertMtx.at<float>(3,1) = 0.042509331921002;
  ConvertMtx.at<float>(3,2) = -0.037318244967432;
  #endif

  //matrix from cvx 
  #if 1
  ConvertMtx.at<float>(0,0) = 0.000167179131;
  ConvertMtx.at<float>(0,1) = 0.021378405283;
  ConvertMtx.at<float>(0,2) = -0.000600300420;
  ConvertMtx.at<float>(1,0) = 0.000037285102;
  ConvertMtx.at<float>(1,1) = 0.017335413184;
  ConvertMtx.at<float>(1,2) = -0.000607142696;
  ConvertMtx.at<float>(2,0) = -0.000384788819;
  ConvertMtx.at<float>(2,1) = 0.011063773501;
  ConvertMtx.at<float>(2,2) = -0.000583754041;
  ConvertMtx.at<float>(3,0) = -0.000211589221;
  ConvertMtx.at<float>(3,1) = 0.018000447150;
  ConvertMtx.at<float>(3,2) = -0.000621909939;
  #endif 

  //calculate the LMSR matrix
  Mat LMSRMtx(4,rows*cols,CV_32FC1); 

  Mat LMSR(4,1,CV_32FC1);
  Mat RGB(3,1,CV_32FC1);

  float minL = FLT_MAX;
  float minM = FLT_MAX;
  float minS = FLT_MAX;
  float minR = FLT_MAX;
  
  //test
  for(int i=0;i<rows;i++) {
    for(int j=0;j<cols;j++) {
      int pixIdx = j+(i*cols); 
      RGB.at<float>(0,0) =buffer[pixIdx*3 +0];
      RGB.at<float>(1,0) =buffer[pixIdx*3 +1];
      RGB.at<float>(2,0) =buffer[pixIdx*3 +2];

      LMSR = ConvertMtx*RGB;
      LMtx->at<float>(i,j) = LMSR.at<float>(0,0); 
      MMtx->at<float>(i,j) = LMSR.at<float>(1,0); 
      SMtx->at<float>(i,j) = LMSR.at<float>(2,0); 
      RMtx->at<float>(i,j) = LMSR.at<float>(3,0); 
      if (LMtx->at<float>(i,j) < minL)
        minL = LMtx->at<float>(i,j);
      if (MMtx->at<float>(i,j) < minM)
        minM = MMtx->at<float>(i,j);
      if (SMtx->at<float>(i,j) < minS)
        minS = SMtx->at<float>(i,j);
      if (RMtx->at<float>(i,j) < minR)
        minR = RMtx->at<float>(i,j);
    }
  }
  printf("min LMSR values are : %f %f %f %f\n", minL, minM, minS, minR);

  printf("ABSOLUTE MINIMUM IS: %f\n", min(minL,min(minM,min(minS, minR))));

  *LMtx = (*LMtx) + 1.675662963531257 +10.0;
  *MMtx = (*MMtx) + 0.925748855805361 +10.0;
  *SMtx = (*SMtx) + 0.172549864617374 +10.0;
  *RMtx = (*RMtx) + 0.566675161287737 +10.0;
  
  #if 0
  //a method to get rid of negative LMSR values 
  for (int r=0; r< rows; r++){
    for (int c = 0; c< cols; c++){
      LMtx->at<float>(r,c) -= minL; 
      MMtx->at<float>(r,c) -= minM;
      SMtx->at<float>(r,c) -= minS;
      RMtx->at<float>(r,c) -= minR;
    }
  }
  #endif
}


#if 0
void HDRToLMSR(const char* filename, Mat* LMtx, Mat* MMtx, Mat* SMtx, Mat* RMtx)
{
  FILE *f = fopen(filename, "rb");
  float * buffer;
  int rows,cols;

  if (f == NULL) {fputs ("File error", stderr); exit(1);}

  //obtain file size:
  fscanf(f,"%d\n%d\n",&cols,&rows);
  
  //read in buffers
  buffer = new float[rows*cols*3];
  fread(buffer,sizeof(float),rows*cols*3,f);
  fclose(f);
 
  //for blending with final output.
  WriteExrRGB("Result/tonemappedHDR.exr",buffer,cols,rows);
  int i = system("./../TMlib/FredoToneMap/tone_mapping Result/tonemappedHDR.exr Result/tonemappedHDR.ppm 50.0");  
  printf("system call returned %d\n", i);

  //create convert matrix
  Mat ConvertMtx(4,3,CV_32FC1);

  #if 0
  //without white balance
  ConvertMtx.at<float>(0,0) = 0.023287629229077;
  ConvertMtx.at<float>(0,1) = 0.031761158107049;
  ConvertMtx.at<float>(0,2) = -0.065877379943775;
  ConvertMtx.at<float>(1,0) = 0.017477925227959;
  ConvertMtx.at<float>(1,1) = 0.025045276134633;
  ConvertMtx.at<float>(1,2) = -0.049961881755050;
  ConvertMtx.at<float>(2,0) = 0.005286774425047;
  ConvertMtx.at<float>(2,1) = 0.011053248400830;
  ConvertMtx.at<float>(2,2) = -0.014602394968991;
  ConvertMtx.at<float>(3,0) = 0.014179531162352;
  ConvertMtx.at<float>(3,1) = 0.023900602623377;
  ConvertMtx.at<float>(3,2) = -0.041615950732866;
  #endif

  //applied white balance on HDR training images 
  ConvertMtx.at<float>(0,0) = 0.053423145735707;
  ConvertMtx.at<float>(0,1) = 0.054289560383423;
  ConvertMtx.at<float>(0,2) = -0.061136577556301;
  ConvertMtx.at<float>(1,0) = 0.039629141289410;
  ConvertMtx.at<float>(1,1) = 0.043169600891946;
  ConvertMtx.at<float>(1,2) = -0.046254335314552;
  ConvertMtx.at<float>(2,0) = 0.007918710531123;
  ConvertMtx.at<float>(2,1) = 0.020833170138768;
  ConvertMtx.at<float>(2,2) = -0.011451393239790;
  ConvertMtx.at<float>(3,0) = 0.029447978916113;
  ConvertMtx.at<float>(3,1) = 0.042509331921002;
  ConvertMtx.at<float>(3,2) = -0.037318244967432;

  //calculate the LMSR matrix
  Mat LMSRMtx(4,rows*cols,CV_32FC1); 

  Mat LMSR(4,1,CV_32FC1);
  Mat RGB(3,1,CV_32FC1);
  
  //test
  #if 0
  float magicNumber = 0.1;
  for(int i=0;i<rows;i++) 
  {
    for(int j=0;j<cols;j++) 
    {
      int pixIdx = j+(i*cols); 
      RGB.at<float>(0,0) =buffer[pixIdx*3 +0];// * magicNumber;
      RGB.at<float>(1,0) =buffer[pixIdx*3 +1];// * magicNumber;
      RGB.at<float>(2,0) =buffer[pixIdx*3 +2];// * magicNumber;

      LMSR = ConvertMtx*RGB;
      LMtx->at<float>(i,j) = LMSR.at<float>(0,0) * magicNumber;
      MMtx->at<float>(i,j) = LMSR.at<float>(1,0) * magicNumber;
      SMtx->at<float>(i,j) = LMSR.at<float>(2,0) * magicNumber;
      RMtx->at<float>(i,j) = LMSR.at<float>(3,0) * magicNumber;
    }
  }
  #endif
  float minL = FLT_MAX;
  float minM = FLT_MAX;
  float minS = FLT_MAX;
  float minR = FLT_MAX;
  
  //test
  float magicNumber = 0.1;
  for(int i=0;i<rows;i++) {
    for(int j=0;j<cols;j++) {
      int pixIdx = j+(i*cols); 
      RGB.at<float>(0,0) =buffer[pixIdx*3 +0];// * magicNumber;
      RGB.at<float>(1,0) =buffer[pixIdx*3 +1];// * magicNumber;
      RGB.at<float>(2,0) =buffer[pixIdx*3 +2];// * magicNumber;

      LMSR = ConvertMtx*RGB;
      LMtx->at<float>(i,j) = LMSR.at<float>(0,0); //* magicNumber;
      MMtx->at<float>(i,j) = LMSR.at<float>(1,0); //* magicNumber;
      SMtx->at<float>(i,j) = LMSR.at<float>(2,0); //* magicNumber;
      RMtx->at<float>(i,j) = LMSR.at<float>(3,0); //* magicNumber;
      if (LMtx->at<float>(i,j) < minL)
        minL = LMtx->at<float>(i,j);
      if (MMtx->at<float>(i,j) < minM)
        minM = MMtx->at<float>(i,j);
      if (SMtx->at<float>(i,j) < minS)
        minS = SMtx->at<float>(i,j);
      if (RMtx->at<float>(i,j) < minR)
        minR = RMtx->at<float>(i,j);
    }
  }

  *LMtx = (*LMtx)-minL;
  *MMtx = (*MMtx)-minM;
  *SMtx = (*SMtx)-minS;
  *RMtx = (*RMtx)-minR;
}
#endif
