#include "lms2display.h"
#include "utilities.h"

using namespace cv;
using namespace std;

//input: lms matrix is a three channel 32-bit floating point image
//return: reshaped color matrix (outImage), where monitorLMS*outImage = colors
void lms2display(Mat* lms, Mat* outImage)
{
  int rows = lms->rows;
  int cols = lms->cols;

  Mat colors(3, rows*cols,CV_32FC1);
  for (int r = 0; r < rows; r++){
    for (int c = 0; c < cols; c++)
    {
      colors.at<float>(0,c+cols*r) = lms->at<Vec3f>(r,c)[0];
      colors.at<float>(1,c+cols*r) = lms->at<Vec3f>(r,c)[1];
      colors.at<float>(2,c+cols*r) = lms->at<Vec3f>(r,c)[2];
    }
  }

  //monitorLMS matrix based on Adam's monitor settings
  Mat monitorLMS(3,3,CV_32FC1);
  monitorLMS.at<float>(0,0) = 3.737001334053387; 
  monitorLMS.at<float>(0,1) = 4.474289710481138; 
  monitorLMS.at<float>(0,2) = 0.446763557036099; 
  monitorLMS.at<float>(1,0) = 1.260813115018955; 
  monitorLMS.at<float>(1,1) = 4.394344052644990; 
  monitorLMS.at<float>(1,2) = 0.587779102275466; 
  monitorLMS.at<float>(2,0) = 0.046245589359130; 
  monitorLMS.at<float>(2,1) = 0.244453440088045; 
  monitorLMS.at<float>(2,2) = 1.218879359722785; 

  Mat color(3, rows*cols,CV_32FC1);
  color = monitorLMS.inv() * colors;

  //reshape color matrix
  for (int i = 0; i < rows; i++)
  {
    for (int j = 0; j < cols; j++)
    {
      //clamp the negative values to 0
      if (color.at<float>(0,j+cols*i) < 0.0f)  color.at<float>(0,j+cols*i) = 0.0f; 
      if (color.at<float>(1,j+cols*i) < 0.0f)  color.at<float>(1,j+cols*i) = 0.0f; 
      if (color.at<float>(2,j+cols*i) < 0.0f)  color.at<float>(2,j+cols*i) = 0.0f; 
      outImage->at<Vec3f>(i,j)[0] = color.at<float>(0,j+cols*i); 
      outImage->at<Vec3f>(i,j)[1] = color.at<float>(1,j+cols*i); 
      outImage->at<Vec3f>(i,j)[2] = color.at<float>(2,j+cols*i); 
    }
  }
} 
