#include "hdrToLMSR.h"
#include "purkinje.h"
#include "lms2display.h"
#include "tonemapping.h"
#include "reduceRange.h"
#include "utilities.h"
#include <iostream>
#include <string>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

using namespace cv;
using namespace std;

/*
***************************OUR PIPELINE*************************** 
1. Read in LDR (8-bits RGB) image
2. Inverse gamma (optional)
3. Divide pixel values by exposure setting to 
approximate HDR pixel values. Also apply a multiplier 
from user to account for desired brightness in final output image.
4. Use the new matrix mentioned earlier to approximate LMSR by HDR RGB
5. For each one of LMSR channels, apply an offset to make all values 
in the channel non-negative. Note this offset is then different
for different channels or images.
6. Run purkinje tone mapping
7. Run reduce range
8. Blend original LDR and output of step 8 to create final output
*******************************************************************
*/

#if 0
//read in video
int main(int argc, char *argv[]) 
{
  MyTimer timer;
  timer.Start();

  const char* videoName;
  double exposureReduce;
  double frameRate;
  double widthVideo;
  double heightVideo;
  double currentFrame = 0;
  double ratio=0;
  int cols, rows;
  IplImage *img;
  Mat *inputMat;
  Mat *hdrImage;
  int counter = 0;
  int numFrames =0;

  if (argc < 2){
    cout<<"**************************************************************************"<<endl;
    cout<<"usage: "<< argv[0] <<" <video name>"<<" <exposure reduce ratio [0.0,1.0]>"<<endl;
    cout<<"**************************************************************************"<<endl;
    return 0;
  }
  else { 
    videoName = argv[1];
  }
  
  if (argc > 2) {
    exposureReduce = atof(argv[2]);
  }

  if (argc >3) {
    numFrames = atoi(argv[3]);
  }

  CvCapture* capture = cvCaptureFromAVI(videoName);

  //Query the properties of this video
  frameRate =  cvGetCaptureProperty(capture, CV_CAP_PROP_FPS); //fram erate
  printf("frame rate of this video is %f\n", frameRate);
  widthVideo = cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_WIDTH);
  printf("width of this video is %f\n", widthVideo);
  heightVideo = cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_HEIGHT);
  printf("height of this video is %f\n", heightVideo);
  cols = widthVideo;  rows = heightVideo;

  hdrImage   = new Mat(heightVideo, widthVideo, CV_32FC3);
  Mat *LMtx  = new Mat(rows,cols,CV_32FC1); 
  Mat *MMtx  = new Mat(rows,cols,CV_32FC1); 
  Mat *SMtx  = new Mat(rows,cols,CV_32FC1); 
  Mat *RMtx  = new Mat(rows,cols,CV_32FC1);  
  Mat* lms   = new Mat(rows, cols, CV_32FC3);
  Mat* blend = new Mat(rows,cols,CV_32FC1);
  Mat *lms2displayOutput = new Mat(rows,cols,CV_32FC3); 

  while (1) {
    img = cvQueryFrame(capture);
    if (!img) break;
    //convert img to Mat.
    inputMat = new Mat(img);
    //convert 8-bit image to 32 bit image
    inputMat->convertTo(*hdrImage,CV_32FC3,exposureReduce*64.0,0.0);      

    //convert hdr to lmsr
    HDRToLMSR(hdrImage,LMtx,MMtx,SMtx,RMtx);
    purkinje(LMtx,MMtx,SMtx,RMtx,lms,blend);    
    lms2display(lms,lms2displayOutput);
    tonemapping(lms2displayOutput);     
    Mat reduceRangeInput = imread("Result/AfterDurandTonemapping.ppm",1); 

    //construct output frame name
    //string outputImgName = "Output/frame";
    //outputImgName.append(counter);
    //outputImgName.append(".png");

    std::stringstream ss;
    ss << "newOutput/frame";
    ss << counter;
    ss << ".png";
    string fileName = ss.str();

    printf("outputImgName is %s\n", fileName.c_str());
    reduceRange(&reduceRangeInput,blend,rows,cols,fileName.c_str(),*inputMat); 
    counter ++;

    if (counter ==numFrames) 
      break;
    //display video
    //cvShowImage("input", img);
    //char c = cvWaitKey(33);
    //if (c == 27) break;
  }
  cvReleaseCapture(&capture);
  timer.Stop();
  timer.Print("total time: ");
}
#endif

#if 1
//read in ldr
int main(int argc, char *argv[]) 
{
  int cols, rows;
  Mat ldrImage = imread(argv[1], 1);
  cols = ldrImage.cols;
  rows = ldrImage.rows;

  Mat *LMtx = new Mat(rows,cols,CV_32FC1); 
  Mat *MMtx = new Mat(rows,cols,CV_32FC1); 
  Mat *SMtx = new Mat(rows,cols,CV_32FC1); 
  Mat *RMtx = new Mat(rows,cols,CV_32FC1);  
  Mat *hdrImage = new Mat(rows,cols, CV_32FC3);
  Mat* lms   = new Mat(rows, cols, CV_32FC3);
  Mat* blend = new Mat(rows,cols,CV_32FC1);
  Mat *lms2displayOutput = new Mat(rows,cols,CV_32FC3); 

  float scaleToHDR = atof(argv[2]);

  ldrImage.convertTo(*hdrImage,CV_32FC3,scaleToHDR,0.0);  
    
  //convert hdr to lmsr
  HDRToLMSR(hdrImage,LMtx,MMtx,SMtx,RMtx);
  purkinje(LMtx,MMtx,SMtx,RMtx,lms,blend);    
  lms2display(lms,lms2displayOutput);
  tonemapping(lms2displayOutput);     
  Mat reduceRangeInput = imread("Result/AfterDurandTonemapping.ppm",1); 

  //construct output frame name
  //string outputImgName = "Output/frame";
  //outputImgName.append(counter);
  //outputImgName.append(".png");

  std::stringstream ss;
  ss << "newOutput/frame";
  ss << 0;
  ss << ".png";
  string fileName = ss.str();

  printf("outputImgName is %s\n", fileName.c_str());
  reduceRange(&reduceRangeInput,blend,rows,cols,fileName.c_str(),ldrImage); 
}
#endif

/*
//read hdr
int main(int argc, char *argv[]) 
{
  MyTimer timer;
  timer.Start();
  int cols, rows;
  const char* HDR_filename = "/Users/yeonjinlee/cs294finalproj/trunk/tools/input/HDRData/troll_hdr_i.bin";
  FILE *f = fopen(HDR_filename, "rb");
  fscanf(f,"%d\n%d\n",&cols,&rows);
  fclose(f);

  Mat *LMtx = new Mat(rows,cols,CV_32FC1); 
  Mat *MMtx = new Mat(rows,cols,CV_32FC1); 
  Mat *SMtx = new Mat(rows,cols,CV_32FC1); 
  Mat *RMtx = new Mat(rows,cols,CV_32FC1);  

  HDRToLMSR(HDR_filename,LMtx,MMtx,SMtx,RMtx);

  Mat* lms = new Mat(rows, cols, CV_32FC3);
  Mat* blend = new Mat(rows,cols,CV_32FC1);
  purkinje(LMtx,MMtx,SMtx,RMtx,lms,blend);
 
  Mat *lms2displayOutput = new Mat(rows,cols,CV_32FC3);

  lms2display(lms,lms2displayOutput);
  
  tonemapping(lms2displayOutput); 

  Mat reduceRangeInput = imread("Result/AfterDurandTonemapping.ppm",1); 
  reduceRange(&reduceRangeInput,blend,rows,cols,"Result/output.png"); 
  timer.Stop();
  timer.Print("total time:");
}
*/
