#include "purkinje.h"
#include "utilities.h"

using namespace cv;
using namespace std;

void purkinje(Mat* LMtx, Mat* MMtx, Mat* SMtx, Mat* RMtx, Mat* I, Mat* blend)
{
  float lmax, mmax, smax, k1, k2, k5, k3PC, k3KC, rw, k6, p;
  lmax=.637;
  mmax=.392;
  smax=1.606;
  k1=.33;
  k2=.5;
  k5=.25;
  k3PC=.8;
  k3KC=.6;
  rw=.139;
  k6=.4;
  p=.6189;

  int rows = LMtx->rows;
  int cols = LMtx->cols;
  int r,c;

  Mat LCopy = LMtx->clone();
  Mat MCopy = MMtx->clone();
  Mat SCopy = SMtx->clone();

  for (r = 0; r < rows; r++)
  {
    for (c = 0; c < cols; c++)
    {
      LCopy.at<float>(r,c) = LMtx->at<float>(r,c) +k5 * RMtx->at<float>(r,c);
      MCopy.at<float>(r,c) = MMtx->at<float>(r,c) +k5 * RMtx->at<float>(r,c);
      SCopy.at<float>(r,c) = SMtx->at<float>(r,c) +k6 * RMtx->at<float>(r,c);
    }
  }
  
  Mat* GL = new Mat(rows,cols,CV_32FC1);
  Mat* GM = new Mat(rows,cols,CV_32FC1);
  Mat* GS = new Mat(rows,cols,CV_32FC1);
  G(&LCopy,k1,k2,GL); 
  G(&MCopy,k1,k2,GM); 
  G(&SCopy,k1,k2,GS);

  Mat* M_L = new Mat(rows,cols,CV_32FC1);
  Mat* S_L_M = new Mat(rows,cols,CV_32FC1);
  Mat* LtM = new Mat(rows, cols, CV_32FC1);
  float a = 15*k5;
  float b = 1+rw*k3PC;
  float d = k3PC+rw;
 
  for (r = 0; r < rows; r++)
  {
    for (c = 0; c < cols; c++)
    {
      M_L->at<float>(r,c) = a*(b *GM->at<float>(r,c) /mmax - d *GL->at<float>(r,c) /lmax);    
      S_L_M->at<float>(r,c) = 15*(k6 * GS->at<float>(r,c)/smax - k3KC* (p*k5*GL->at<float>(r,c)/lmax + (1-p)*k5*GM->at<float>(r,c)/mmax));
      LtM->at<float>(r,c) = 5*k5*(p*GL->at<float>(r,c)/lmax + (1-p)*GM->at<float>(r,c)/mmax); 
 
      blend->at<float>(r,c) = 1 - LtM->at<float>(r,c);
      if (blend->at<float>(r,c) < 0) blend->at<float>(r,c) = 0;
      M_L->at<float>(r,c) = RMtx->at<float>(r,c) * M_L->at<float>(r,c);
      S_L_M->at<float>(r,c) = RMtx->at<float>(r,c) * S_L_M->at<float>(r,c);
      LtM->at<float>(r,c) = RMtx->at<float>(r,c) * LtM->at<float>(r,c);
      float value = (LtM->at<float>(r,c) - M_L->at<float>(r,c))/2;
      I->at<Vec3f>(r,c)[0] = value; 
      I->at<Vec3f>(r,c)[1] = LtM->at<float>(r,c) - value;
      I->at<Vec3f>(r,c)[2] = S_L_M->at<float>(r,c) + LtM->at<float>(r,c);

      I->at<Vec3f>(r,c)[0] += LMtx->at<float>(r,c);
      I->at<Vec3f>(r,c)[1] += MMtx->at<float>(r,c);
      I->at<Vec3f>(r,c)[2] += SMtx->at<float>(r,c);
    }
  }

}

void G(Mat* signal, float k1, float k2, Mat* val)
{
  int rows, cols;
  rows = signal->rows;
  cols = signal->cols;

  for (int r = 0; r < rows; r++)
  {
    for (int c = 0; c < cols; c++)
    {
      val->at<float>(r,c) = 1/ (pow((1 +k1 *signal->at<float>(r,c)) ,k2)); 
    }
  }
} 
