#include "reduceRange.h"
#include "tonemapping.h"
#include "utilities.h"

using namespace cv;
using namespace std;

//input is inputData, a float array returned from tonemapping.cpp
//returns outputImag, which is an unsigned 8-bit RGB image.
void reduceRange(Mat* img, Mat* alpha, int rows, int cols, const char* outputFileName, Mat &LDRImg)
{
  int width = cols;
  int height = rows;
  assert(img->type() == CV_8UC3);

  float x = 0.1f;
  Mat* Idouble = new Mat(rows, cols, CV_32FC3);
  Mat* MaxImage = new Mat(rows,cols,CV_32FC1);
  Mat* alphaScaling = new Mat(rows, cols, CV_32FC1);
  Mat* outputMtx = new Mat(rows, cols, CV_8UC3);
  
  //for blending with the input.
  //Mat LDRImg = imread("Result/tonemappedHDR.ppm",1);  
  float alphaSum =0;
  for (int r = 0; r < rows; r++){
    for (int c = 0; c < cols; c++)
    {
      Idouble->at<Vec3f>(r,c)[0] = ((double) img->at<Vec3b>(r,c)[0])/256.0;
      Idouble->at<Vec3f>(r,c)[1] = ((double) img->at<Vec3b>(r,c)[1])/256.0;
      Idouble->at<Vec3f>(r,c)[2] = ((double) img->at<Vec3b>(r,c)[2])/256.0;
      MaxImage->at<float>(r,c) = 255.0;
      alphaScaling->at<float>(r,c) = (alpha->at<float>(r,c)*(1-x))+x;
      MaxImage->at<float>(r,c) *= alphaScaling->at<float>(r,c);
    
      int pixelIdx = c+(r*cols);
      outputMtx->at<Vec3b>(r,c)[0] = (unsigned int)(Idouble->at<Vec3f>(r,c)[0]*MaxImage->at<float>(r,c));
      outputMtx->at<Vec3b>(r,c)[1] = (unsigned int)(Idouble->at<Vec3f>(r,c)[1]*MaxImage->at<float>(r,c));
      outputMtx->at<Vec3b>(r,c)[2] = (unsigned int)(Idouble->at<Vec3f>(r,c)[2]*MaxImage->at<float>(r,c));

      //blending
      float colorBlendStart = 210.0;
      float intensity = 0.0, bright_factor = 0.0;
      intensity = 0.3 * (float) LDRImg.at<Vec3b>(r,c)[0] +  0.6 * (float) LDRImg.at<Vec3b>(r,c)[1] + 0.1 * (float) LDRImg.at<Vec3b>(r,c)[2];
      if (intensity >= 210.0) {
              bright_factor = (intensity - colorBlendStart) / (float) (255 - colorBlendStart);
      }
      float finalBlendR = (1-bright_factor)*alpha->at<float>(r,c) + bright_factor;
      float finalBlendG = (1-bright_factor)*alpha->at<float>(r,c) + bright_factor;
      float finalBlendB = (1-bright_factor)*alpha->at<float>(r,c) + bright_factor;	
		
	#if 0
      float purkinjeVsFixedR = ((float) LDRImg.at<Vec3b>(r,c)[0] - colorBlendStart)/ (255.0f - colorBlendStart);
      float purkinjeVsFixedG = ((float) LDRImg.at<Vec3b>(r,c)[1] - colorBlendStart)/ (255.0f - colorBlendStart);
      float purkinjeVsFixedB = ((float) LDRImg.at<Vec3b>(r,c)[2] - colorBlendStart)/ (255.0f - colorBlendStart);
      if (purkinjeVsFixedR <0) purkinjeVsFixedR = 0;
      if (purkinjeVsFixedG <0) purkinjeVsFixedG = 0;
      if (purkinjeVsFixedB <0) purkinjeVsFixedB = 0;
      float finalBlendR = (1-purkinjeVsFixedR)*alpha->at<float>(r,c) + purkinjeVsFixedR*1.0f;
      float finalBlendG = (1-purkinjeVsFixedG)*alpha->at<float>(r,c) + purkinjeVsFixedG*1.0f;
      float finalBlendB = (1-purkinjeVsFixedB)*alpha->at<float>(r,c) + purkinjeVsFixedB*1.0f;
   #endif   
      outputMtx->at<Vec3b>(r,c)[0] =  (unsigned int)(finalBlendR * LDRImg.at<Vec3b>(r,c)[0] + ((1- finalBlendR) * outputMtx->at<Vec3b>(r,c)[0]));
      outputMtx->at<Vec3b>(r,c)[1] =  (unsigned int)(finalBlendG * LDRImg.at<Vec3b>(r,c)[1] + ((1- finalBlendG) * outputMtx->at<Vec3b>(r,c)[1]));
      outputMtx->at<Vec3b>(r,c)[2] =  (unsigned int)(finalBlendB * LDRImg.at<Vec3b>(r,c)[2] + ((1- finalBlendB) * outputMtx->at<Vec3b>(r,c)[2]));
      alphaSum += alpha->at<float>(r,c);
    }
  }

  float alphaMean = alphaSum/(rows*cols);
  printf("The average blend value is %f\n", alphaMean);

  imwrite(outputFileName,*outputMtx);
}
