#include "rgbToLMSR.h"

using namespace cv;
using namespace std;

void ProcessVideo()
{
  CvCapture* capture = cvCaptureFromAVI("/Users/yeonjinlee/Desktop/vtfire.avi");
  
  double frameRate;
  double widthVideo;
  double heightVideo;
  double currentFrame =0;
  double ratio=0;
  IplImage* img;
  cvNamedWindow("input");
  cvNamedWindow("output");

  //Query the properties of this video
  frameRate =  cvGetCaptureProperty(capture, CV_CAP_PROP_FPS); //fram erate
  printf("frame rate of this video is %f\n", frameRate);
  widthVideo = cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_WIDTH);
  printf("width of this video is %f\n", widthVideo);
  heightVideo = cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_HEIGHT);
  printf("height of this video is %f\n", heightVideo);

  //display the frames 
  while (1)
  {
    img = cvQueryFrame(capture); //grab and retrieve.
    if (!img) break;
    //do image processing here.
    cvShowImage("input",img);
    //create output image
    IplImage* out = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 3);
    //do gaussian blur
    cvSmooth(img,out,CV_GAUSSIAN,11,11);
    cvShowImage("output",out);
    char c = cvWaitKey(33); //wait 33ms before displaying frame
    if (c ==27) break;
  }

  cvReleaseCapture(&capture);
  cvDestroyWindow("output");
  cvDestroyWindow("image");
}

void RGBToLMSR(Mat* image, Mat* LMtx, Mat* MMtx, Mat* SMtx, Mat* RMtx)
{
  int rows, cols, totalPixels;
  rows = image->rows;
  cols = image->cols;
  
  //Image is 3 channel float?
  CV_Assert(image->type() == CV_8UC3);
  
  //create a RGB matrix (6291456x3)
  totalPixels = rows*cols;
  
  //convert the input image from 8 bits (0->255) to float (0->1)
  //image->convertTo(image,CV_32FC3, 1/255.0,0.0);  

  Mat RGBMtx(3,totalPixels,CV_32FC1); 
  //contstruct the RGB image matrix
  for (int i = 0; i<rows; i++)
  {
    for (int j = 0; j<cols; j++)
    {
      const Vec3b& rgb = image->at<Vec3b>(i,j); 
      RGBMtx.at<float>(0,(i*cols)+j) = rgb[0]/255.0; //rgb is 8UC, which goes from 1-255. Need to be 32FC, which is (0->1)
      RGBMtx.at<float>(1,(i*cols)+j) = rgb[1]/255.0;
      RGBMtx.at<float>(2,(i*cols)+j) = rgb[2]/255.0;
    }
  }
  
  //create convert matrix
  Mat ConvertMtx(4,3,CV_32FC1);
  ConvertMtx.at<float>(0,0) = 0.096869562190332;
  ConvertMtx.at<float>(0,1) = 0.318940374720484;
  ConvertMtx.at<float>(0,2) = -0.188428411786113;
  ConvertMtx.at<float>(1,0) = 0.020208210904239;
  ConvertMtx.at<float>(1,1) = 0.291385283197581;
  ConvertMtx.at<float>(1,2) = -0.090918262127325;
  ConvertMtx.at<float>(2,0) = 0.002760510899553;
  ConvertMtx.at<float>(2,1) = -0.008341563564118;
  ConvertMtx.at<float>(2,2) = 0.067213551661950;
  ConvertMtx.at<float>(3,0) = -0.007607045462440;
  ConvertMtx.at<float>(3,1) = 0.122492925567539;
  ConvertMtx.at<float>(3,2) = 0.022445835141881;
  
  //calculate the LMSR matrix
  Mat LMSRMtx(4,rows*cols,CV_32FC1); 
  LMSRMtx = ConvertMtx*RGBMtx;

  //LMSRMtx dimension is (4 x(rows*cols)). Put this into 4 images of size cols x rows
  //Mat LMtx(cols,rows,CV_32FC1); 
  //Mat MMtx(cols,rows,CV_32FC1); 
  //Mat SMtx(cols,rows,CV_32FC1); 
  //Mat RMtx(cols,rows,CV_32FC1); 

  for(int i=0;i<rows;i++) 
  {
    for(int j=0;j<cols;j++) 
    {
      LMtx->at<float>(i,j) = LMSRMtx.at<float>(0,j + i*cols);
      MMtx->at<float>(i,j) = LMSRMtx.at<float>(1,j + i*cols);
      SMtx->at<float>(i,j) = LMSRMtx.at<float>(2,j + i*cols);
      RMtx->at<float>(i,j) = LMSRMtx.at<float>(3,j + i*cols);
    } 
  }
}
